# Consumidor de Datos de Kafka para el Cálculo Diario de Casos de COVID-19 para los Últimos 30 Días

## Descripción

Este consumidor de Kafka fue creado como parte de la plataforma EpI-PUMA y tiene como objetivo proveer una base de datos que contenga únicamente los casos confirmados de COVID-19 en México para los últimos 30 días agrupados por fecha, municipio, grupo de edad y tipo de caso.

La base de datos generada por este conector fue pensada como una herramienta para demostrar la flexibilidad del sistema Apache Kafka, así como para facilitar la comparación de las predicciones realizadas por los modelos estadísticos de la plataforma EpI-PUMA con los datos reales de los casos de COVID-19 en México.

## Base de Datos

Este conector genera una base de datos de PostgreSQL de manera automática. La base de datos generada posee la siguiente estructura:

![Screenshot](datos/database.png)

## Requisitos

Para poder ejecutar el proceso de carga de datos en Kafka es necesario tener instalados los siguientes paquetes en el sistema donde se ejecutará el proceso:
  - Python 3 y Pip

Además, en el mismo sistema o en uno dedicado se debe tener instalado y configurado un servidor de Apache Kafka con un tópico alimentado por el productor [KafkaCovid19](https://git.c3.unam.mx/alonso.ballesteros/KafkaCovid19).


## Instalación

Basta con ejecutar el siguiente comando para instalar las librerías necesarias para ejecutar el proceso de carga de datos en Kafka:

```sh
pip install -r requirements.txt
```

## Configuración

Los ajustes del conector se realizan en el archivo **conector.py**. Estos ajustes deben realizarse antes de ejecutar el conector, ya que de lo contrario fallará con una excepción.

### Base de datos de SQLite

- Es simplemente el nombre del archivo que se usará para guardar la base de datos local. Esta base se emplea como un buffer para guardar los registros que se reciben de Kafka antes de guardarlos en la base de datos de PostgreSQL.
- No es necesario crear el archivo de la base de datos, puesto que el conector lo creará de manera automática.

```python
# ----------- Datos SQLite ------------
SQ_ARCHIVO = 'datos.db'
```

### Base de datos de PostgreSQL

- Debe ser una base de datos de PostgreSQL completamente vacía, puesto que el conector creará las tablas necesarias de manera automática.

```python
# --------- Datos PostgreSQL ----------
PG_HOST       = 'base.datos.com'
PG_PUERTO     = 5433
PG_USUARIO    = 'usuario'
PG_CLAVE      = 'clave'
PG_BASE_DATOS = 'epi_puma'
```

### Apache Kafka

- En el campo de **KAFKA_BOOTSTRAP_SERVER** se debe omitir el protocolo **http://**.
- El campo **KAFKA_TOPIC** debe coincidir con el nombre del tópico que alimenta el productor [KafkaCovid19](https://git.c3.unam.mx/alonso.ballesteros/KafkaCovid19).
- Se sugiere que el campo **KAFKA_GROUP_ID** sea único para este consumidor de Kafka.

```python
# ------------ Datos Kafka ------------
KAFKA_BOOTSTRAP_SERVER = 'servidor.kafka:9092'
KAFKA_SCHEMA_REGISTRY  = 'http://servidor.kafka:8081'
KAFKA_GROUP_ID         = 'covid19_epi_puma_30d'
KAFKA_TOPIC            = 'covid19'
KAFKA_PARTITION        = 0
```

### Conector

- El campo **ARCHIVO_FECHA** denota el nombre del archivo que se usará para guardar la fecha del último registro que se guardó en la base de datos de PostgreSQL. Se recomienda no modificar este campo.

- El campo **FECHA_INICIAL** denota la fecha a partir de la cual se comenzarán a guardar los registros en la base de datos de PostgreSQL. El valor se expresa en el formato **AAAA-MM-DD**.

- El campo **TIEMPO_PURGA_REGISTROS** denota el tiempo que esperará el consumidor sin recibir nuevos registros antes de guardar los registros que tiene en memoria en la base de datos. El valor se expresa en segundos.

- El campo **LIMITE_CARGA_REGISTROS** denota el número máximo de registros que se guardarán en memoria antes de hacerlos persistentes en la base de datos, esto se realiza de manera transaccional. El valor se expresa en número de registros.

```python
ARCHIVO_FECHA = 'fecha.txt'
FECHA_INICIAL = '2021-03-01'
TIEMPO_PURGA_REGISTROS = 60
LIMITE_CARGA_REGISTROS = 100000
```

## Conector

El conector posee una interfaz de línea de comandos que permite ejecutarlo de manera sencilla. La interfaz de línea de comandos se muestra a continuación:

```sh
python3 conector.py [-h] {estatus,iniciar,detener}

positional arguments:
  {estatus,iniciar,detener}

optional arguments:
  -h, --help            show this help message and exit
```

## Ejecución

Para ejecutar el consumidor de Kafka basta con ingresar el siguiente comando:

```sh
python3 conector.py iniciar
```

Con este comando el consumidor de Kafka se ejecutará en segundo plano y se mantendrá en ejecución hasta que se ingrese el siguiente comando:

```sh
python3 conector.py detener
```

Además, se puede obtener el estatus del consumidor de Kafka con el siguiente comando:

```sh
python3 conector.py estatus
```

El conector guardará un archivo de registros en el archivo **conector.log**. Este archivo se encuentra en el directorio donde se ejecuta el conector.